@@ -34,7 +34,7 @@
 #include "callbacks.h"
 #include "dialogs.h"
 #include "documentprivate.h"
-#include "encodings.h"
+#include "encodingsprivate.h"
 #include "filetypes.h"
 #include "geanymenubuttonaction.h"
 #include "keyfile.h"
@@ -512,10 +512,19 @@ void ui_update_popup_goto_items(gboolean enable)
 }
 
 
+void ui_menu_copy_items_set_sensitive(gboolean sensitive)
+{
+	guint i, len;
+
+	len = G_N_ELEMENTS(widgets.menu_copy_items);
+	for (i = 0; i < len; i++)
+		ui_widget_set_sensitive(widgets.menu_copy_items[i], sensitive);
+}
+
+
 void ui_update_menu_copy_items(GeanyDocument *doc)
 {
 	gboolean enable = FALSE;
-	guint i, len;
 	GtkWidget *focusw = gtk_window_get_focus(GTK_WINDOW(main_widgets.window));
 
 	g_return_if_fail(doc == NULL || doc->is_valid);
@@ -533,9 +542,7 @@ void ui_update_menu_copy_items(GeanyDocument *doc)
 		enable = gtk_text_buffer_get_selection_bounds(buffer, NULL, NULL);
 	}
 
-	len = G_N_ELEMENTS(widgets.menu_copy_items);
-	for (i = 0; i < len; i++)
-		ui_widget_set_sensitive(widgets.menu_copy_items[i], enable);
+	ui_menu_copy_items_set_sensitive(enable);
 }
 
 
@@ -608,8 +615,6 @@ static void on_menu_insert_include_activate(GtkMenuItem *menuitem, gpointer user
 static void insert_include_items(GtkMenu *me, GtkMenu *mp, gchar **includes, gchar *label)
 {
 	guint i = 0;
-	GtkWidget *tmp_menu;
-	GtkWidget *tmp_popup;
 	GtkWidget *edit_menu, *edit_menu_item;
 	GtkWidget *popup_menu, *popup_menu_item;
 
@@ -622,8 +627,9 @@ static void insert_include_items(GtkMenu *me, GtkMenu *mp, gchar **includes, gch
 
 	while (includes[i] != NULL)
 	{
-		tmp_menu = gtk_menu_item_new_with_label(includes[i]);
-		tmp_popup = gtk_menu_item_new_with_label(includes[i]);
+		GtkWidget *tmp_menu = gtk_menu_item_new_with_label(includes[i]);
+		GtkWidget *tmp_popup = gtk_menu_item_new_with_label(includes[i]);
+
 		gtk_container_add(GTK_CONTAINER(edit_menu), tmp_menu);
 		gtk_container_add(GTK_CONTAINER(popup_menu), tmp_popup);
 		g_signal_connect(tmp_menu, "activate",
@@ -875,7 +881,7 @@ static void init_document_widgets(void)
 	add_doc_widget("add_comments1");
 	add_doc_widget("menu_paste1");
 	add_doc_widget("menu_undo2");
-	add_doc_widget("preferences2");
+	add_doc_widget("properties1");
 	add_doc_widget("menu_reload1");
 	add_doc_widget("menu_document1");
 	add_doc_widget("menu_choose_color1");
@@ -1106,16 +1112,15 @@ void ui_set_search_entry_background(GtkWidget *widget, gboolean success)
 
 static void recent_create_menu(GeanyRecentFiles *grf)
 {
-	GtkWidget *tmp;
 	guint i, len;
-	gchar *filename;
 
 	len = MIN(file_prefs.mru_length, g_queue_get_length(grf->recent_queue));
 	for (i = 0; i < len; i++)
 	{
-		filename = g_queue_peek_nth(grf->recent_queue, i);
 		/* create menu item for the recent files menu in the menu bar */
-		tmp = gtk_menu_item_new_with_label(filename);
+		const gchar *filename = g_queue_peek_nth(grf->recent_queue, i);
+		GtkWidget *tmp = gtk_menu_item_new_with_label(filename);
+
 		gtk_widget_show(tmp);
 		gtk_container_add(GTK_CONTAINER(grf->menubar), tmp);
 		g_signal_connect(tmp, "activate", G_CALLBACK(grf->activate_cb), NULL);
@@ -1461,7 +1466,12 @@ void ui_update_view_editor_menu_items(void)
 /** Creates a GNOME HIG-style frame (with no border and indented child alignment).
  * @param label_text The label text.
  * @param alignment An address to store the alignment widget pointer.
- * @return The frame widget, setting the alignment container for packing child widgets. */
+ *
+ * @return @transfer{floating} The frame widget, setting the alignment container for
+ * packing child widgets.
+ * 
+ * @deprecated 1.29: Use GTK API directly
+ **/
 GEANY_API_SYMBOL
 GtkWidget *ui_frame_new_with_alignment(const gchar *label_text, GtkWidget **alignment)
 {
@@ -1484,7 +1494,8 @@ GtkWidget *ui_frame_new_with_alignment(const gchar *label_text, GtkWidget **alig
 
 /** Makes a fixed border for dialogs without increasing the button box border.
  * @param dialog The parent container for the @c GtkVBox.
- * @return The packed @c GtkVBox. */
+ *
+ * @return @transfer{none} The packed @c GtkVBox. */
 GEANY_API_SYMBOL
 GtkWidget *ui_dialog_vbox_new(GtkDialog *dialog)
 {
@@ -1535,7 +1546,8 @@ void ui_dialog_set_primary_button_order(GtkDialog *dialog, gint response, ...)
  * @c gtk_button_new_from_stock().
  * @param stock_id A @c GTK_STOCK_NAME string.
  * @param text Button label text, can include mnemonics.
- * @return The new @c GtkButton.
+ *
+ * @return @transfer{floating} The new @c GtkButton.
  */
 GEANY_API_SYMBOL
 GtkWidget *ui_button_new_with_image(const gchar *stock_id, const gchar *text)
@@ -1554,7 +1566,7 @@ GtkWidget *ui_button_new_with_image(const gchar *stock_id, const gchar *text)
 /** Creates a @c GtkImageMenuItem with a stock image and a custom label.
  * @param stock_id Stock image ID, e.g. @c GTK_STOCK_OPEN.
  * @param label Menu item label, can include mnemonics.
- * @return The new @c GtkImageMenuItem.
+ * @return @transfer{floating} The new @c GtkImageMenuItem.
  *
  *  @since 0.16
  */
@@ -1680,7 +1692,7 @@ static gboolean tree_model_find_text(GtkTreeModel *model,
 /** Prepends @a text to the drop down list, removing a duplicate element in
  * the list if found. Also ensures there are <= @a history_len elements.
  * @param combo_entry .
- * @param text Text to add, or @c NULL for current entry text.
+ * @param text @nullable Text to add, or @c NULL for current entry text.
  * @param history_len Max number of items, or @c 0 for default. */
 GEANY_API_SYMBOL
 void ui_combo_box_add_to_history(GtkComboBoxText *combo_entry,
@@ -1894,31 +1906,30 @@ void ui_widget_modify_font_from_string(GtkWidget *widget, const gchar *str)
  * file chooser, replacing entry text (if successful) with the path returned from the
  * @c GtkFileChooser.
  * @note @a entry can be the child of an unparented widget, such as @c GtkComboBoxEntry.
- * @param title The file chooser dialog title, or @c NULL.
+ * @param title @nullable The file chooser dialog title, or @c NULL.
  * @param action The mode of the file chooser.
  * @param entry Can be an unpacked @c GtkEntry, or the child of an unpacked widget,
  * such as @c GtkComboBoxEntry.
- * @return The @c GtkHBox.
+ *
+ * @return @transfer{floating} The @c GtkHBox.
  */
 /* @see ui_setup_open_button_callback(). */
 GEANY_API_SYMBOL
 GtkWidget *ui_path_box_new(const gchar *title, GtkFileChooserAction action, GtkEntry *entry)
 {
-	GtkWidget *vbox, *dirbtn, *openimg, *hbox, *path_entry;
+	GtkWidget *vbox, *dirbtn, *openimg, *hbox, *path_entry, *parent, *next_parent;
 
 	hbox = gtk_hbox_new(FALSE, 6);
 	path_entry = GTK_WIDGET(entry);
 
 	/* prevent path_entry being vertically stretched to the height of dirbtn */
 	vbox = gtk_vbox_new(FALSE, 0);
-	if (gtk_widget_get_parent(path_entry))	/* entry->parent may be a GtkComboBoxEntry */
-	{
-		GtkWidget *parent = gtk_widget_get_parent(path_entry);
 
-		gtk_box_pack_start(GTK_BOX(vbox), parent, TRUE, FALSE, 0);
-	}
-	else
-		gtk_box_pack_start(GTK_BOX(vbox), path_entry, TRUE, FALSE, 0);
+	parent = path_entry;
+	while ((next_parent = gtk_widget_get_parent(parent)) != NULL)
+		parent = next_parent;
+
+	gtk_box_pack_start(GTK_BOX(vbox), parent, TRUE, FALSE, 0);
 
 	dirbtn = gtk_button_new();
 	openimg = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
@@ -2471,10 +2482,28 @@ void ui_init_builder(void)
 static void init_custom_style(void)
 {
 #if GTK_CHECK_VERSION(3, 0, 0)
-	gchar *css_file = g_build_filename(app->datadir, "geany.css", NULL);
+	const struct {
+		guint version;
+		const gchar *file;
+	} css_files[] = {
+		/*
+		 * Keep these from newest to oldest,
+		 * and make sure 0 remains last
+		 */
+		{ 20, "geany-3.20.css" },
+		{ 0, "geany-3.0.css" },
+	};
+	guint gtk_version = gtk_get_minor_version();
+	gsize i = 0;
+	gchar *css_file;
 	GtkCssProvider *css = gtk_css_provider_new();
 	GError *error = NULL;
 
+	/* gtk_version will never be smaller than 0 */
+	while (css_files[i].version > gtk_version)
+		++i;
+
+	css_file = g_build_filename(app->datadir, css_files[i].file, NULL);
 	if (! gtk_css_provider_load_from_path(css, css_file, &error))
 	{
 		g_warning("Failed to load custom CSS: %s", error->message);
@@ -2651,7 +2680,8 @@ void ui_widget_set_tooltip_text(GtkWidget *widget, const gchar *text)
  * you want returned.
  * @param widget Widget with the @a widget_name property set.
  * @param widget_name Name to lookup.
- * @return The widget found.
+ *
+ * @return @transfer{none} The widget found.
  * @see ui_hookup_widget().
  *
  *  @since 0.16
@@ -2731,7 +2761,7 @@ static gboolean progress_bar_pulse(gpointer data)
  * In this case, you need to show and hide the widget yourself. You can find some example code
  * in @c src/printing.c.
  *
- * @param text The text to be shown as the progress bar label or NULL to leave it empty.
+ * @param text @nullable The text to be shown as the progress bar label or @c NULL to leave it empty.
  *
  *  @since 0.16
  **/
@@ -2814,7 +2844,7 @@ void ui_label_set_markup(GtkLabel *label, const gchar *format, ...)
 	gchar *text;
 
 	va_start(a, format);
-	text = g_strdup_vprintf(format, a);
+	text = g_markup_vprintf_escaped(format, a);
 	va_end(a);
 
 	gtk_label_set_text(label, text);
@@ -2826,23 +2856,22 @@ void ui_label_set_markup(GtkLabel *label, const gchar *format, ...)
 GtkWidget *ui_label_new_bold(const gchar *text)
 {
 	GtkWidget *label;
-	gchar *label_text;
 
-	label_text = g_markup_escape_text(text, -1);
 	label = gtk_label_new(NULL);
-	ui_label_set_markup(GTK_LABEL(label), "<b>%s</b>", label_text);
-	g_free(label_text);
+	ui_label_set_markup(GTK_LABEL(label), "<b>%s</b>", text);
 	return label;
 }
 
 
-/** Adds a list of document items to @a menu.
+/** @girskip
+ * Adds a list of document items to @a menu.
  * @param menu Menu.
- * @param active Which document to highlight, or @c NULL.
- * @param callback is used for each menu item's @c "activate" signal and will be passed
- * the corresponding document pointer as @c user_data.
+ * @param active @nullable Which document to highlight, or @c NULL.
+ * @param callback is used for each menu item's @c "activate" signal and will be
+ * passed the corresponding document pointer as @c user_data.
  * @warning You should check @c doc->is_valid in the callback.
- * @since 0.19 */
+ * @since 0.19
+ **/
 GEANY_API_SYMBOL
 void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback callback)
 {
@@ -2850,7 +2879,8 @@ void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback
 }
 
 
-/** Adds a list of document items to @a menu.
+/** @girskip
+ * Adds a list of document items to @a menu.
  *
  * @a compare_func might be NULL to not sort the documents in the menu. In this case,
  * the order of the document tabs is used.
@@ -2858,12 +2888,13 @@ void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback
  * See document_compare_by_display_name() for an example sort function.
  *
  * @param menu Menu.
- * @param active Which document to highlight, or @c NULL.
+ * @param active @nullable Which document to highlight, or @c NULL.
  * @param callback is used for each menu item's @c "activate" signal and will be passed
  * the corresponding document pointer as @c user_data.
  * @param compare_func is used to sort the list. Might be @c NULL to not sort the list.
  * @warning You should check @c doc->is_valid in the callback.
- * @since 0.21 */
+ * @since 0.21
+ **/
 GEANY_API_SYMBOL
 void ui_menu_add_document_items_sorted(GtkMenu *menu, GeanyDocument *active,
 	GCallback callback, GCompareFunc compare_func)
@@ -2871,7 +2902,7 @@ void ui_menu_add_document_items_sorted(GtkMenu *menu, GeanyDocument *active,
 	GtkWidget *menu_item, *menu_item_label, *image;
 	GeanyDocument *doc;
 	guint i, len;
-	gchar *base_name, *label;
+	gchar *base_name;
 	GPtrArray *sorted_documents;
 
 	len = (guint) gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
@@ -2905,11 +2936,7 @@ void ui_menu_add_document_items_sorted(GtkMenu *menu, GeanyDocument *active,
 		gtk_widget_set_name(menu_item_label, document_get_status_widget_class(doc));
 
 		if (doc == active)
-		{
-			label = g_markup_escape_text(base_name, -1);
-			ui_label_set_markup(GTK_LABEL(menu_item_label), "<b>%s</b>", label);
-			g_free(label);
-		}
+			ui_label_set_markup(GTK_LABEL(menu_item_label), "<b>%s</b>", base_name);
 
 		g_free(base_name);
 	}
